@@ -1,4 +1,4 @@
-/*	$NetBSD: arch.c,v 1.26 1997/10/03 17:58:00 thorpej Exp $	*/
+/*	$NetBSD: arch.c,v 1.29 1998/11/11 19:37:06 christos Exp $	*/
 
 /*
  * Copyright (c) 1988, 1989, 1990, 1993
@@ -39,14 +39,14 @@
  */
 
 #ifdef MAKE_BOOTSTRAP
-static char rcsid[] = "$NetBSD: arch.c,v 1.26 1997/10/03 17:58:00 thorpej Exp $";
+static char rcsid[] = "$NetBSD: arch.c,v 1.29 1998/11/11 19:37:06 christos Exp $";
 #else
 #include <sys/cdefs.h>
 #ifndef lint
 #if 0
 static char sccsid[] = "@(#)arch.c	8.2 (Berkeley) 1/2/94";
 #else
-__RCSID("$NetBSD: arch.c,v 1.26 1997/10/03 17:58:00 thorpej Exp $");
+__RCSID("$NetBSD: arch.c,v 1.29 1998/11/11 19:37:06 christos Exp $");
 #endif
 #endif /* not lint */
 #endif
@@ -114,6 +114,15 @@ __RCSID("$NetBSD: arch.c,v 1.26 1997/10/03 17:58:00 thorpej Exp $");
 #include    "dir.h"
 #include    "config.h"
 
+#ifdef TARGET_MACHINE
+#undef MACHINE
+#define MACHINE TARGET_MACHINE
+#endif
+#ifdef TARGET_MACHINE_ARCH
+#undef MACHINE_ARCH
+#define MACHINE_ARCH TARGET_MACHINE_ARCH
+#endif
+
 static Lst	  archives;   /* Lst of archives we've already examined */
 
 typedef struct Arch {
@@ -130,7 +139,7 @@ static struct ar_hdr *ArchStatMember __P((char *, char *, Boolean));
 static FILE *ArchFindMember __P((char *, char *, struct ar_hdr *, char *));
 #if defined(__svr4__) || defined(__SVR4) || defined(__ELF__)
 #define SVR4ARCHIVES
-static int ArchSVR4Entry __P((Arch *, char *, size_t, FILE *));
+static int ArchSVR4Entry __P((Arch *, char **, size_t, FILE *));
 #endif
 
 /*-
@@ -243,11 +252,11 @@ Arch_ParseArchive (linePtr, nodeLst, ctxt)
 	 */
 	Boolean	doSubst = FALSE; /* TRUE if need to substitute in memName */
 
-	while (*cp != '\0' && *cp != ')' && isspace (*cp)) {
+	while (*cp != '\0' && *cp != ')' && isspace ((unsigned char)*cp)) {
 	    cp++;
 	}
 	memName = cp;
-	while (*cp != '\0' && *cp != ')' && !isspace (*cp)) {
+	while (*cp != '\0' && *cp != ')' && !isspace ((unsigned char)*cp)) {
 	    if (*cp == '$') {
 		/*
 		 * Variable spec, so call the Var module to parse the puppy
@@ -412,7 +421,7 @@ Arch_ParseArchive (linePtr, nodeLst, ctxt)
      */
     do {
 	cp++;
-    } while (*cp != '\0' && isspace (*cp));
+    } while (*cp != '\0' && isspace ((unsigned char)*cp));
 
     *linePtr = cp;
     return (SUCCESS);
@@ -474,7 +483,7 @@ ArchStatMember (archive, member, hash)
     Arch	  *ar;	      /* Archive descriptor */
     Hash_Entry	  *he;	      /* Entry containing member's description */
     struct ar_hdr arh;        /* archive-member header for reading archive */
-    char	  memName[MAXPATHLEN+1];
+    char	  *memName = NULL; 
     	    	    	    /* Current member name while hashing. */
 
     /*
@@ -556,7 +565,6 @@ ArchStatMember (archive, member, hash)
     ar->fnametab = NULL;
     ar->fnamesize = 0;
     Hash_InitTable (&ar->members, -1);
-    memName[AR_MAX_NAME_LEN] = '\0';
 
     while (fread ((char *)&arh, sizeof (struct ar_hdr), 1, arch) == 1) {
 	if (strncmp ( arh.ar_fmag, ARFMAG, sizeof (arh.ar_fmag)) != 0) {
@@ -575,10 +583,9 @@ ArchStatMember (archive, member, hash)
 	    arh.ar_size[sizeof(arh.ar_size)-1] = '\0';
 	    size = (int) strtol(arh.ar_size, NULL, 10);
 
-	    (void) strncpy (memName, arh.ar_name, sizeof(arh.ar_name));
-	    for (cp = &memName[AR_MAX_NAME_LEN]; *cp == ' '; cp--) {
-		continue;
-	    }
+	    memName = estrdup(arh.ar_name);
+
+	    for(cp = strchr(memName, 0)-1; *cp == ' '; cp--);
 	    cp[1] = '\0';
 
 #ifdef SVR4ARCHIVES
@@ -589,7 +596,7 @@ ArchStatMember (archive, member, hash)
 		/*
 		 * svr4 magic mode; handle it
 		 */
-		switch (ArchSVR4Entry(ar, memName, size, arch)) {
+		switch (ArchSVR4Entry(ar, &memName, size, arch)) {
 		case -1:  /* Invalid data */
 		    goto badarch;
 		case 0:	  /* List of files entry */
@@ -614,8 +621,8 @@ ArchStatMember (archive, member, hash)
 
 		unsigned int elen = atoi(&memName[sizeof(AR_EFMT1)-1]);
 
-		if (elen > MAXPATHLEN)
-			goto badarch;
+		memName = erealloc(memName, elen+1);
+		
 		if (fread (memName, elen, 1, arch) != 1)
 			goto badarch;
 		memName[elen] = '\0';
@@ -631,6 +638,9 @@ ArchStatMember (archive, member, hash)
 	    memcpy ((Address)Hash_GetValue (he), (Address)&arh,
 		sizeof (struct ar_hdr));
 	}
+	/* time to free memName */
+	free(memName);
+
 	fseek (arch, (size + 1) & ~1, SEEK_CUR);
     }
 
@@ -683,7 +693,7 @@ badarch:
 static int
 ArchSVR4Entry(ar, name, size, arch)
 	Arch *ar;
-	char *name;
+	char **name;
 	size_t size;
 	FILE *arch;
 {
@@ -692,8 +702,8 @@ ArchSVR4Entry(ar, name, size, arch)
     size_t entry;
     char *ptr, *eptr;
 
-    if (strncmp(name, ARLONGNAMES1, sizeof(ARLONGNAMES1) - 1) == 0 ||
-	strncmp(name, ARLONGNAMES2, sizeof(ARLONGNAMES2) - 1) == 0) {
+    if (strncmp(*name, ARLONGNAMES1, sizeof(ARLONGNAMES1) - 1) == 0 ||
+	strncmp(*name, ARLONGNAMES2, sizeof(ARLONGNAMES2) - 1) == 0) {
 
 	if (ar->fnametab != NULL) {
 	    if (DEBUG(ARCH)) {
@@ -736,30 +746,30 @@ ArchSVR4Entry(ar, name, size, arch)
 	return 0;
     }
 
-    if (name[1] == ' ' || name[1] == '\0')
+    if ((*name)[1] == ' ' || (*name)[1] == '\0')
 	return 2;
 
-    entry = (size_t) strtol(&name[1], &eptr, 0);
-    if ((*eptr != ' ' && *eptr != '\0') || eptr == &name[1]) {
+    entry = (size_t) strtol(&(*name)[1], &eptr, 0);
+    if ((*eptr != ' ' && *eptr != '\0') || eptr == &(*name)[1]) {
 	if (DEBUG(ARCH)) {
-	    printf("Could not parse SVR4 name %s\n", name);
+	    printf("Could not parse SVR4 name %s\n", *name);
 	}
 	return 2;
     }
     if (entry >= ar->fnamesize) {
 	if (DEBUG(ARCH)) {
 	    printf("SVR4 entry offset %s is greater than %lu\n",
-		   name, (u_long)ar->fnamesize);
+		   *name, (u_long)ar->fnamesize);
 	}
 	return 2;
     }
 
     if (DEBUG(ARCH)) {
-	printf("Replaced %s with %s\n", name, &ar->fnametab[entry]);
+	printf("Replaced %s with %s\n", *name, &ar->fnametab[entry]);
     }
 
-    (void) strncpy(name, &ar->fnametab[entry], MAXPATHLEN);
-    name[MAXPATHLEN] = '\0';
+		free(*name);
+		*name = estrdup(&ar->fnametab[entry]);
     return 1;
 }
 #endif
@@ -866,12 +876,9 @@ ArchFindMember (archive, member, arhPtr, mode)
 		isdigit(arhPtr->ar_name[sizeof(AR_EFMT1) - 1])) {
 
 		unsigned int elen = atoi(&arhPtr->ar_name[sizeof(AR_EFMT1)-1]);
-		char ename[MAXPATHLEN];
+		char *ename = NULL;
 
-		if (elen > MAXPATHLEN) {
-			fclose (arch);
-			return NULL;
-		}
+		ename = emalloc(elen+1);
 		if (fread (ename, elen, 1, arch) != 1) {
 			fclose (arch);
 			return NULL;
@@ -885,6 +892,8 @@ ArchFindMember (archive, member, arhPtr, mode)
 			fseek (arch, -sizeof(struct ar_hdr) - elen, SEEK_CUR);
 			return (arch);
 		}
+		/* time to free ename */
+		free(ename);
 		fseek (arch, -elen, SEEK_CUR);
 		goto skip;
 	} else
@@ -1068,11 +1077,11 @@ Arch_MemMTime (gn)
 	    nameStart = strchr (pgn->name, '(') + 1;
 	    nameEnd = strchr (nameStart, ')');
 
-	    if (pgn->make &&
+	    if ((pgn->flags & REMAKE) &&
 		strncmp(nameStart, gn->name, nameEnd - nameStart) == 0) {
 				     gn->mtime = Arch_MTime(pgn);
 	    }
-	} else if (pgn->make) {
+	} else if (pgn->flags & REMAKE) {
 	    /*
 	     * Something which isn't a library depends on the existence of
 	     * this target, so it needs to exist.
