@@ -420,7 +420,7 @@ CompatMake (gnp, pgnp)
 	    }
 	    return (0);
 	} else if (DEBUG(MAKE)) {
-	    printf("out-of-date.\n");
+	    printf("out-of-date\n");
 	}
 
 	/*
@@ -556,7 +556,7 @@ Compat_Run(targs)
 	signal(SIGQUIT, CompatInterrupt);
     }
 
-    for (cp = "#=|^(){};&<>*?[]:$`\\\n"; *cp != '\0'; cp++) {
+    for (cp = "~#=|^(){};&<>*?[]:$`\\\n"; *cp != '\0'; cp++) {
 	meta[(unsigned char) *cp] = 1;
     }
     /*
