@@ -1,4 +1,4 @@
-/*	$NetBSD: main.c,v 1.39 1997/11/08 09:33:16 lukem Exp $	*/
+/*	$NetBSD: main.c,v 1.45 1998/11/01 03:07:34 itohy Exp $	*/
 
 /*
  * Copyright (c) 1988, 1989, 1990, 1993
@@ -39,7 +39,7 @@
  */
 
 #ifdef MAKE_BOOTSTRAP
-static char rcsid[] = "$NetBSD: main.c,v 1.39 1997/11/08 09:33:16 lukem Exp $";
+static char rcsid[] = "$NetBSD: main.c,v 1.45 1998/11/01 03:07:34 itohy Exp $";
 #else
 #include <sys/cdefs.h>
 #ifndef lint
@@ -51,7 +51,7 @@ __COPYRIGHT("@(#) Copyright (c) 1988, 1989, 1990, 1993\n\
 #if 0
 static char sccsid[] = "@(#)main.c	8.3 (Berkeley) 3/19/94";
 #else
-__RCSID("$NetBSD: main.c,v 1.39 1997/11/08 09:33:16 lukem Exp $");
+__RCSID("$NetBSD: main.c,v 1.45 1998/11/01 03:07:34 itohy Exp $");
 #endif
 #endif /* not lint */
 #endif
@@ -97,6 +97,7 @@ __RCSID("$NetBSD: main.c,v 1.39 1997/11/08 09:33:16 lukem Exp $");
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <time.h>
 #ifdef __STDC__
 #include <stdarg.h>
 #else
@@ -140,7 +141,7 @@ Boolean			mkIncPath;	/* -m flag */
 static Boolean		jobsRunning;	/* TRUE if the jobs might be running */
 
 static void		MainParseArgs __P((int, char **));
-char *			chdir_verify_path __P((char *, char *));
+char *			chdir_verify_path __P((char *));
 static int		ReadMakefile __P((ClientData, ClientData));
 static void		usage __P((void));
 
@@ -171,7 +172,7 @@ MainParseArgs(argc, argv)
 	int c;
 	int forceJobs = 0;
 
-	optind = 1;	/* since we're called more than once */
+	optind = 0;	/* since we're called more than once */
 #ifdef REMOTE
 # define OPTFLAGS "BD:I:L:PSV:d:ef:ij:km:nqrst"
 #else
@@ -377,6 +378,9 @@ Main_ParseArgLine(line)
 {
 	char **argv;			/* Manufactured argument vector */
 	int argc;			/* Number of arguments in argv */
+	char *args;			/* Space used by the args */
+	char *buf, *p1;
+	char *argv0 = Var_Value(".MAKE", VAR_GLOBAL, &p1);
 
 	if (line == NULL)
 		return;
@@ -385,16 +389,25 @@ Main_ParseArgLine(line)
 	if (!*line)
 		return;
 
-	argv = brk_string(line, &argc, TRUE);
+	buf = emalloc(strlen(line) + strlen(argv0) + 2);
+	(void)sprintf(buf, "%s %s", argv0, line);
+	if (p1)
+		free(p1);
+
+	argv = brk_string(buf, &argc, TRUE, &args);
+	free(buf);
 	MainParseArgs(argc, argv);
+
+	free(args);
+	free(argv);
 }
 
 char *
-chdir_verify_path(path, obpath)
+chdir_verify_path(path)
 	char *path;
-	char *obpath;
 {
 	struct stat sb;
+	char *obpath = NULL;
 
 	if (stat(path, &sb) == 0 && S_ISDIR(sb.st_mode)) {
 		if (chdir(path)) {
@@ -404,12 +417,13 @@ chdir_verify_path(path, obpath)
 		}
 		else {
 			if (path[0] != '/') {
-				(void) snprintf(obpath, MAXPATHLEN, "%s/%s",
-						curdir, path);
+				(void) asprintf(&obpath, "%s/%s", curdir, path);
+				if(!obpath)
+					enomem();
 				return obpath;
 			}
 			else
-				return path;
+				return estrdup(path);
 		}
 	}
 
@@ -443,10 +457,8 @@ main(argc, argv)
 	Boolean outOfDate = TRUE; 	/* FALSE if all targets up to date */
 	struct stat sb, sa;
 	char *p, *p1, *path, *pathp, *pwd;
-	char mdpath[MAXPATHLEN + 1];
-	char obpath[MAXPATHLEN + 1];
-	char cdpath[MAXPATHLEN + 1];
-    	char *machine = getenv("MACHINE");
+	char *mdpath = NULL;
+  char *machine = getenv("MACHINE");
 	char *machine_arch = getenv("MACHINE_ARCH");
 	Lst sysMkPath;			/* Path of sys.mk */
 	char *cp = NULL, *start;
@@ -471,8 +483,7 @@ main(argc, argv)
 	 * All this code is so that we know where we are when we start up
 	 * on a different machine with pmake.
 	 */
-	curdir = cdpath;
-	if (getcwd(curdir, MAXPATHLEN) == NULL) {
+	if ((curdir = getcwd(NULL,0)) == NULL) {
 		(void)fprintf(stderr, "make: %s.\n", strerror(errno));
 		exit(2);
 	}
@@ -486,7 +497,8 @@ main(argc, argv)
 	if ((pwd = getenv("PWD")) != NULL) {
 	    if (stat(pwd, &sb) == 0 && sa.st_ino == sb.st_ino &&
 		sa.st_dev == sb.st_dev)
-		(void) strcpy(curdir, pwd);
+		free(curdir);
+		curdir = estrdup(pwd);
 	}
 
 	/*
@@ -513,7 +525,11 @@ main(argc, argv)
 
 	if (!machine_arch) {
 #ifndef MACHINE_ARCH
+#ifdef __ARCHITECTURE__
+            machine_arch = __ARCHITECTURE__;
+#else
 	    machine_arch = "unknown";	/* XXX: no uname -p yet */
+#endif
 #else
 	    machine_arch = MACHINE_ARCH;
 #endif
@@ -536,25 +552,24 @@ main(argc, argv)
 		if (!(path = getenv("MAKEOBJDIR"))) {
 			path = _PATH_OBJDIR;
 			pathp = _PATH_OBJDIRPREFIX;
-			(void) snprintf(mdpath, MAXPATHLEN, "%s.%s",
-					path, machine);
-			if (!(objdir = chdir_verify_path(mdpath, obpath)))
-				if (!(objdir=chdir_verify_path(path, obpath))) {
-					(void) snprintf(mdpath, MAXPATHLEN,
-							"%s%s", pathp, curdir);
-					if (!(objdir=chdir_verify_path(mdpath,
-								       obpath)))
+			(void) asprintf(&mdpath, "%s.%s", path, machine);
+			if (!(objdir = chdir_verify_path(mdpath)))
+				if (!(objdir=chdir_verify_path(path))) {
+					(void) asprintf(&mdpath, "%s%s", pathp, curdir);
+					if (!(objdir=chdir_verify_path(mdpath)))
 						objdir = curdir;
 				}
 		}
-		else if (!(objdir = chdir_verify_path(path, obpath)))
+		else if (!(objdir = chdir_verify_path(path)))
 			objdir = curdir;
 	}
 	else {
-		(void) snprintf(mdpath, MAXPATHLEN, "%s%s", pathp, curdir);
-		if (!(objdir = chdir_verify_path(mdpath, obpath)))
+		(void) asprintf(&mdpath, "%s%s", pathp, curdir);
+		if (!(objdir = chdir_verify_path(mdpath)))
 			objdir = curdir;
 	}
+	/* time to free mdpath */
+	free(mdpath);
 
 	setenv("PWD", objdir, 1);
 
@@ -599,7 +614,6 @@ main(argc, argv)
 				 * directories */
 	Var_Init();		/* As well as the lists of variables for
 				 * parsing arguments */
-	str_init();
 	Var_Set(".CURDIR", curdir, VAR_GLOBAL);
 	Var_Set(".OBJDIR", objdir, VAR_GLOBAL);
 
@@ -610,6 +624,7 @@ main(argc, argv)
 	 *	MFLAGS also gets initialized empty, for compatibility.
 	 */
 	Var_Set("MAKE", argv[0], VAR_GLOBAL);
+	Var_Set(".MAKE", argv[0], VAR_GLOBAL);
 	Var_Set(MAKEFLAGS, "", VAR_GLOBAL);
 	Var_Set("MFLAGS", "", VAR_GLOBAL);
 	Var_Set("MACHINE", machine, VAR_GLOBAL);
@@ -820,10 +835,10 @@ main(argc, argv)
 	Suff_End();
         Targ_End();
 	Arch_End();
-	str_end();
 	Var_End();
 	Parse_End();
 	Dir_End();
+	Job_End();
 
 	if (queryFlag && outOfDate)
 		return(1);
@@ -848,7 +863,7 @@ ReadMakefile(p, q)
 	char *fname = p;		/* makefile to read */
 	extern Lst parseIncPath;
 	FILE *stream;
-	char *name, path[MAXPATHLEN + 1];
+	char *name, *path;
 
 	if (!strcmp(fname, "-")) {
 		Parse_File("(stdin)", stdin);
@@ -858,7 +873,7 @@ ReadMakefile(p, q)
 			goto found;
 		/* if we've chdir'd, rebuild the path name */
 		if (curdir != objdir && *fname != '/') {
-			(void)sprintf(path, "%s/%s", curdir, fname);
+			(void)asprintf(&path, "%s/%s", curdir, fname);
 			if ((stream = fopen(path, "r")) != NULL) {
 				fname = path;
 				goto found;
@@ -881,6 +896,9 @@ found:		Var_Set("MAKEFILE", fname, VAR_GLOBAL);
 		(void)fclose(stream);
 	}
 	return(TRUE);
+	/* `path' is not freed before return because its
+	 * pointer is stored somewhere else
+	 */
 }
 
 /*-
@@ -996,9 +1014,9 @@ Cmd_Exec(cmd, err)
 	 * install it in the variable.
 	 */
 	res[cc] = '\0';
-	cp = &res[cc] - 1;
+	cp = &res[cc];
 
-	if (*cp == '\n') {
+	if (cc > 0 && *--cp == '\n') {
 	    /*
 	     * A final newline is just stripped
 	     */
