@@ -183,104 +183,7 @@ strrcpy(ptr, str)
     return (ptr);
 } /* end strrcpy */
 
-
-char   *
-getwd(pathname)
-    char   *pathname;
-{
-    DIR    *dp;
-    struct dirent *d;
-    extern int errno;
-
-    struct stat st_root, st_cur, st_next, st_dotdot;
-    char    pathbuf[MAXPATHLEN], nextpathbuf[MAXPATHLEN * 2];
-    char   *pathptr, *nextpathptr, *cur_name_add;
-
-    /* find the inode of root */
-    if (stat("/", &st_root) == -1) {
-	(void) sprintf(pathname,
-			"getwd: Cannot stat \"/\" (%s)", strerror(errno));
-	return (NULL);
-    }
-    pathbuf[MAXPATHLEN - 1] = '\0';
-    pathptr = &pathbuf[MAXPATHLEN - 1];
-    nextpathbuf[MAXPATHLEN - 1] = '\0';
-    cur_name_add = nextpathptr = &nextpathbuf[MAXPATHLEN - 1];
-
-    /* find the inode of the current directory */
-    if (lstat(".", &st_cur) == -1) {
-	(void) sprintf(pathname,
-			"getwd: Cannot stat \".\" (%s)", strerror(errno));
-	return (NULL);
-    }
-    nextpathptr = strrcpy(nextpathptr, "../");
-
-    /* Descend to root */
-    for (;;) {
-
-	/* look if we found root yet */
-	if (st_cur.st_ino == st_root.st_ino &&
-	    DEV_DEV_COMPARE(st_cur.st_dev, st_root.st_dev)) {
-	    (void) strcpy(pathname, *pathptr != '/' ? "/" : pathptr);
-	    return (pathname);
-	}
-
-	/* open the parent directory */
-	if (stat(nextpathptr, &st_dotdot) == -1) {
-	    (void) sprintf(pathname,
-			    "getwd: Cannot stat directory \"%s\" (%s)",
-			    nextpathptr, strerror(errno));
-	    return (NULL);
-	}
-	if ((dp = opendir(nextpathptr)) == NULL) {
-	    (void) sprintf(pathname,
-			    "getwd: Cannot open directory \"%s\" (%s)",
-			    nextpathptr, strerror(errno));
-	    return (NULL);
-	}
-
-	/* look in the parent for the entry with the same inode */
-	if (DEV_DEV_COMPARE(st_dotdot.st_dev, st_cur.st_dev)) {
-	    /* Parent has same device. No need to stat every member */
-	    for (d = readdir(dp); d != NULL; d = readdir(dp))
-		if (d->d_fileno == st_cur.st_ino)
-		    break;
-	}
-	else {
-	    /*
-	     * Parent has a different device. This is a mount point so we
-	     * need to stat every member
-	     */
-	    for (d = readdir(dp); d != NULL; d = readdir(dp)) {
-		if (ISDOT(d->d_name) || ISDOTDOT(d->d_name))
-		    continue;
-		(void) strcpy(cur_name_add, d->d_name);
-		if (lstat(nextpathptr, &st_next) == -1) {
-		    (void) sprintf(pathname, "getwd: Cannot stat \"%s\" (%s)",
-				    d->d_name, strerror(errno));
-		    (void) closedir(dp);
-		    return (NULL);
-		}
-		/* check if we found it yet */
-		if (st_next.st_ino == st_cur.st_ino &&
-		    DEV_DEV_COMPARE(st_next.st_dev, st_cur.st_dev))
-		    break;
-	    }
-	}
-	if (d == NULL) {
-	    (void) sprintf(pathname, "getwd: Cannot find \".\" in \"..\"");
-	    (void) closedir(dp);
-	    return (NULL);
-	}
-	st_cur = st_dotdot;
-	pathptr = strrcpy(pathptr, d->d_name);
-	pathptr = strrcpy(pathptr, "/");
-	nextpathptr = strrcpy(nextpathptr, "../");
-	(void) closedir(dp);
-	*cur_name_add = '\0';
-    }
-} /* end getwd */
-
+/* `getwd()' was used nowhere in the code, was removed */
 
 char    *sys_siglist[] = {
         "Signal 0",
