@@ -1,4 +1,4 @@
-/*	$NetBSD: var.c,v 1.23 1997/09/28 03:31:14 lukem Exp $	*/
+/*	$NetBSD: var.c,v 1.30 1998/11/06 23:31:09 christos Exp $	*/
 
 /*
  * Copyright (c) 1988, 1989, 1990, 1993
@@ -39,14 +39,14 @@
  */
 
 #ifdef MAKE_BOOTSTRAP
-static char rcsid[] = "$NetBSD: var.c,v 1.23 1997/09/28 03:31:14 lukem Exp $";
+static char rcsid[] = "$NetBSD: var.c,v 1.30 1998/11/06 23:31:09 christos Exp $";
 #else
 #include <sys/cdefs.h>
 #ifndef lint
 #if 0
 static char sccsid[] = "@(#)var.c	8.3 (Berkeley) 3/19/94";
 #else
-__RCSID("$NetBSD: var.c,v 1.23 1997/09/28 03:31:14 lukem Exp $");
+__RCSID("$NetBSD: var.c,v 1.30 1998/11/06 23:31:09 christos Exp $");
 #endif
 #endif /* not lint */
 #endif
@@ -95,6 +95,8 @@ __RCSID("$NetBSD: var.c,v 1.23 1997/09/28 03:31:14 lukem Exp $");
 
 #include    <ctype.h>
 #ifndef MAKE_BOOTSTRAP
+#include    <stdio.h>
+#include    <sys/types.h>
 #include    <regex.h>
 #endif
 #include    <stdlib.h>
@@ -606,6 +608,7 @@ Var_Value (name, ctxt, frp)
     if (v != (Var *) NIL) {
 	char *p = ((char *)Buf_GetAll(v->val, (int *)NULL));
 	if (v->flags & VAR_FROM_ENV) {
+	    free(v->name);
 	    Buf_Destroy(v->val, FALSE);
 	    free((Address) v);
 	    *frp = p;
@@ -1156,7 +1159,8 @@ VarRESubstitute(word, addSpace, buf, patternp)
 		Buf_AddByte(buf,rp[1]);
 		rp++;
 	    }
-	    else if ((*rp == '&') || ((*rp == '\\') && isdigit(rp[1]))) {
+	    else if ((*rp == '&') ||
+		((*rp == '\\') && isdigit((unsigned char)rp[1]))) {
 		int n;
 		char *subbuf;
 		int sublen;
@@ -1255,16 +1259,20 @@ VarModify (str, modProc, datum)
 				     * buffer before adding the trimmed
 				     * word */
     char **av;			    /* word list [first word does not count] */
+    char *as;			    /* word list memory */
     int ac, i;
 
     buf = Buf_Init (0);
     addSpace = FALSE;
 
-    av = brk_string(str, &ac, FALSE);
+    av = brk_string(str, &ac, FALSE, &as);
 
-    for (i = 1; i < ac; i++)
+    for (i = 0; i < ac; i++)
 	addSpace = (*modProc)(av[i], addSpace, buf, datum);
 
+    free(as);
+    free(av);
+
     Buf_AddByte (buf, '\0');
     str = (char *)Buf_GetAll (buf, (int *)NULL);
     Buf_Destroy (buf, FALSE);
@@ -1279,7 +1287,7 @@ VarModify (str, modProc, datum)
  *	uninterpreted) and 2) unescaped $'s that aren't before
  *	the delimiter (expand the variable substitution).
  *	Return the expanded string or NULL if the delimiter was missing
- *	If pattern is specified, handle escaped ampersants, and replace
+ *	If pattern is specified, handle escaped ampersands, and replace
  *	unescaped ampersands with the lhs of the pattern.
  *
  * Results:
@@ -1717,6 +1725,9 @@ Var_Parse (str, ctxt, err, lengthPtr, freePtr)
      *  	  	    	each word
      *  	  :R	    	Substitute the root of each word
      *  	  	    	(pathname minus the suffix).
+     *		  :?<true-value>:<false-value>
+     *				If the variable evaluates to true, return
+     *				true value, else return the second value.
      *	    	  :lhs=rhs  	Like :S, but the rhs goes to the end of
      *	    	    	    	the invocation.
      */
@@ -1839,6 +1850,43 @@ Var_Parse (str, ctxt, err, lengthPtr, freePtr)
 		    free(pattern.lhs);
 		    free(pattern.rhs);
 		    break;
+		}	
+		case '?':
+		{
+		    VarPattern 	pattern;
+		    Boolean	value;
+
+		    /* find ':', and then substitute accordingly */
+
+		    pattern.flags = 0;
+
+		    cp = ++tstr;
+		    delim = ':';
+		    if ((pattern.lhs = VarGetPattern(ctxt, err, &cp, delim,
+			NULL, &pattern.leftLen, NULL)) == NULL)
+			goto cleanup;
+
+		    delim = '}';
+		    if ((pattern.rhs = VarGetPattern(ctxt, err, &cp, delim,
+			NULL, &pattern.rightLen, NULL)) == NULL)
+			goto cleanup;
+
+		    termc = *--cp;
+		    delim = '\0';
+		    if (Cond_EvalExpression(1, str, &value, 0) == COND_INVALID){
+			Error("Bad conditional expression `%s' in %s?%s:%s",
+			      str, str, pattern.lhs, pattern.rhs);
+			goto cleanup;
+		    }
+
+		    if (value) {
+			newStr = pattern.lhs;
+			free(pattern.rhs);
+		    } else {
+			newStr = pattern.rhs;
+			free(pattern.lhs);
+		    }
+		    break;
 		}
 #ifndef MAKE_BOOTSTRAP
 		case 'C':
@@ -2076,6 +2124,7 @@ Var_Parse (str, ctxt, err, lengthPtr, freePtr)
 	     */
 	    *freePtr = TRUE;
 	}
+	free(v->name);
 	Buf_Destroy(v->val, destroy);
 	free((Address)v);
     } else if (v->flags & VAR_JUNK) {
@@ -2104,8 +2153,9 @@ cleanup:
     *lengthPtr = cp - start + 1;
     if (*freePtr)
 	free(str);
-    Error("Unclosed substitution for %s (%c missing)",
-	  v->name, delim);
+    if (delim != '\0')
+	Error("Unclosed substitution for %s (%c missing)",
+	      v->name, delim);
     return (var_Error);
 }
 
